import json
from .Text2TextGen import GPTJ
from .security import check_for_insults


class GPTJ_API(GPTJ):

    def __init__(self, context, user_bot_input_output_example):
        super().__init__(user_bot_input_output_example)
        self.conversation_dictionary = {}
        self.setting = check_for_insults(str(context))
        self.dialog = ""
        self.user_bot_input_output_example = user_bot_input_output_example
        self.Person_speaking = [str(check_for_insults(key)) for key in user_bot_input_output_example]
        self.Bot_responding = [str(check_for_insults(value)) for value in user_bot_input_output_example.values()]
        self.ID = ""
        self.your_friend = ""
        self.json_conversation = {}
        self.intentions = ""
        self.responses = ""
        self.final_intention = []
        self.final_responses = []
        self.all_conversation = []
        self.user_aspect = []
        self.bot_aspect = []
        self.lenght = 0
        self.formatted_convo = []
        self.model_input = []

    def imaginary_friend(self, user_id, friend):
        self.ID = str(user_id) + ": "
        self.your_friend = str(friend) + ": "
        self.dialog = self.gptj_examples(self.Person_speaking, self.Bot_responding)
        self.json_conversation = json.loads(self.dialog)
        self.intentions = self.json_conversation["intentions"]
        self.responses = self.json_conversation["responses"]
        self.final_intention = [self.ID + i for i in self.intentions]
        self.final_responses = [self.your_friend + r for r in self.responses]
        self.all_conversation = [self.final_intention, self.final_responses]
        self.user_aspect = ["".join(i) for i in self.all_conversation[0]]
        self.bot_aspect = ["".join(r) for r in self.all_conversation[1]]
        self.lenght = len(self.user_aspect) * 2
        self.formatted_convo = [i for e in zip(self.user_aspect, [r for r in self.bot_aspect] * len(self.user_aspect))
                                for i in e][:self.lenght]
        self.model_input = '\n'.join(map(str, self.formatted_convo))
        self.setting += "\n" + self.model_input
        return self.setting


import json

convo = {}


class GPTJ:

    def __init__(self, user_bot_input_output_example):
        assert isinstance(user_bot_input_output_example, dict), "User bot examples must be a dictionary"
        self.user_input_example = [key for key in user_bot_input_output_example]
        self.bot_output_example = [value for value in user_bot_input_output_example.values()]
        self.user_inputs = []
        self.bot_inputs = []
        self.conversation_json = {}

    def gptj_examples(self, uu, bb):

        full_conversation = [[], []]

        def username(user_response=uu):
            Examples1 = [str(example) for example in user_response]
            return Examples1

        def imaginary_character(bot_response=bb):
            Examples2 = [str(example2) for example2 in bot_response]
            return Examples2

        self.user_inputs = username(self.user_input_example)
        self.bot_inputs = imaginary_character(self.bot_output_example)

        length_of_user_inputs = len(self.user_inputs)
        length_of_bot_outputs = len(self.bot_inputs)
        if length_of_user_inputs == length_of_bot_outputs:
            full_conversation[0].append(self.user_inputs)
            full_conversation[1].append(self.bot_inputs)
            for element in full_conversation:
                for intention in full_conversation[0]:
                    convo['intentions'] = intention
                for response in full_conversation[1]:
                    convo['responses'] = response
            self.conversation_json = json.dumps(convo, indent=4)
            return self.conversation_json
        else:
            if length_of_user_inputs > length_of_bot_outputs:
                print([self.user_inputs[length_of_bot_outputs]])
                del self.user_inputs[length_of_bot_outputs:]
                full_conversation[0].append(self.user_inputs)
                full_conversation[1].append(self.bot_inputs)
                for element in full_conversation:
                    for intention in full_conversation[0]:
                        convo['intentions'] = intention
                    for response in full_conversation[1]:
                        convo['responses'] = response
                self.conversation_json = json.dumps(convo, indent=2)
                return self.conversation_json
            elif length_of_bot_outputs > length_of_user_inputs:
                print([self.bot_inputs[length_of_user_inputs]])
                del self.bot_inputs[length_of_user_inputs:]
                full_conversation[0].append(self.user_inputs)
                full_conversation[1].append(self.bot_inputs)
                for element in full_conversation:
                    for intention in full_conversation[0]:
                        convo['intentions'] = intention
                    for response in full_conversation[1]:
                        convo['responses'] = response
                self.conversation_json = json.dumps(convo, indent=2)
                return self.conversation_json
            return self.conversation_json


from gpt_j.gptj_api import Completion

context = """I am a highly intelligent Python Bot and I can give you a complex and detailed code in Python for your task I also use the power of 3, 6, and 9 to return good and flawless outputs from the quantum field.
My code is ALWAYS properly indented. I always remember to indent my code which I know is a code habit to have. I print only one line of code per line. I use an appropriate amount of comments. 
I import all libraries every time. 
I use comments in the code. I use # in my code when giving feedback.
I return a LONG code that actually works everytime. 
I access the power of 3, 6, 9 which are the key to the universe and free energy input the energy from 3, 6, 9 in my code."""

examples = {
    "Ask user for a number between 1 and 24th prime number. Test if it is a Fibonacci number.":
        """n = int(input('Enter a number between 1 and 89:'))
    if n in [0, 1, 1, 2, 3, 5, 8, 13, 21, 34, 55, 89]:
    print("You entered: ", n) else:   print("That is not a Fibonacci number")""",

    "Calculate the sine value of number stored in num":
        """import math
num = int(input('Enter a number: '))
sin_value = math.sin(num) 
print("The sine of your number is: ", sin_value, ".")""",

    "Print the top and bottom rows of the data frame":
        """import pandas as pd
import numpy as np
df = pd.DataFrame(np.random.randint(1, 10, size=(5, 4)), columns=['a', 'b', 'c', 'd'])
print("The top row and bottom rows are:\m", df.iloc[[0, -1]])
""",

    "make a decision tree classifier on the IRIS dataset":
        """from sklearn import datasets
from sklearn import metrics
from sklearn.tree import DecisionTreeClassifier
dataset = datasets.load_iris()
model = DecisionTreeClassifier()
model.fit(dataset.data, dataset.target)
print(model)
expected = dataset.target
predicted = model.predict(dataset.data)
print(metrics.classification_report(expected, predicted))""",

    "Delete all vowels from input text.":
        """def rem_vowel(string):
    vowels = ['a','e','i','o','u']
    result = [letter for letter in string if letter.lower() not in vowels]
    result = ''.join(result)
    print(result)
string = "test"
rem_vowel(string)
string = "remove vowels here"
rem_vowel(string)
""",

    "Plot sin x":
        """import matplotlib.pyplot as plt
import numpy as np
x = np.linspace(-10, 10, 100)
y = np.sin(x)
plt.plot(x, y)
plt.show()
""",

    "Ask user to enter 3 numbers one by one. Print the product.":

        """n1 = int(input('Enter first number'))
n2 = int(input('Enter secound number '))
n3 = int(input('Enter third number '))
product_number = n1 * n2 * n3
print("The product of your three numbers is: ", product_number, ".")""",

    "Perform a google search of what the user wants and print the top result":

        """import requests
from bs4 import BeautifulSoup
search_url = "https://www.google.com/search?q=" + input('Enter wedsite')
r = requests.get(search_url)
html = r.text
soup = BeautifulSoup(html, 'lxml')
print(soup)""",

    "Print what part of the day is going on right now":

        """import time
mytime = time.localtime()
if mytime.tm_hour < 6 or mytime.tm_hour > 18:
    print ('It is night-time')
else:
    print ('It is day-time')""",

    "Make a password generator":

        """import random
characters = 'abcdefghijklmnopqrstuvwxyz[];\',./{}:\"<>?\\|12345678980!@#$%^&*()-=_+~`'
characters = list(characters)
password = ''
for i in range(0, random.randint(8, 13)):
    char = random.choice(characters)
    password+=char
    print('Your password is:', password)""",

    " Check if the year entered by user is a leap year":

        """import datetime
year = int(input('Enter year'))
if year % 4 == 0 and (year % 100 != 0 or year % 400 == 0):
    print("It is a leap year")
else:
print("It is not a leap year")""",

    "Ask a user to enter 3 numbers one by one. Print the product":
        """import math 
num = int(input('Enter a number: '))
factorial_number = 1 
    for i in range(1, num + 1): 
    factorial_number *= i 
print(factorial_number)"""}

context_setting = Completion(context, examples)

prompt = "Divide A by B stored in a variable result"

User = "Student"

Bot = "Calculator"

temperature = 0.101

top_probability = 0.53

K = 40

seed = 0

stream = True

response = context_setting.completion(prompt,
                                      user=User,
                                      bot=Bot,
                                      temperature=temperature,
                                      top_p=top_probability,
                                      top_k=K,
                                      seed=seed,
                                      stream=stream)

print(response)


import requests
from .security import check_for_insults
from .Gptj import generate


def simple_completion(prompt, temp, top=1.0, top_k=40):
    try:
        behavior = check_for_user_conduct = check_for_insults(prompt)
        assert isinstance(temp, float), "temperature most be float value"
        assert isinstance(top, float), "top_p most be float value"
        assert isinstance(top_k, int), "top_k most be integer value"
        if behavior == "":
            return generate(prompt, temperature=temp, top_p=top, top_k=top_k)
        simple_result = ""
        simple_result += check_for_user_conduct
        return simple_result
    except requests.HTTPError:
        print("""Either the API is down or your values are too high."\nTry keeping max tokens, temperature, and top_p to a reasonable value\nAlso don't add too many examples add enough but not an huge amount""")




from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as gpt:
    long_description = gpt.read()

VERSION = '3.0.5'
DESCRIPTION = 'A GPT-J api to use with python3 to generate text, blogs, code, and more (Note: Starting with version 3.0.0, the API uses a different domain to generate text with GPT-J and features such as max length have been removed)'

setup(name="gptj",
    version=VERSION,
    author="TheProtagonist (Michael Arana)",
    author_email="MichaelGamer256@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/TheProtaganist/gpt-j",
    py_modules=["Basic_api", "Gptj", "gpt_api", "ImaginaryFriend", "security", "Text2TextGen"],
    license="MIT",
    long_description=long_description,
    packages=find_packages(),
    package_dir={" ": "gpt_j"},
    install_requires=['requests', 'ProfanityDetector', 'cryptography'],
    keywords=['python', 'text generation', 'chatbot framework', 'gpt-J', 'gpt-3', 'gpt-2', 'completion', 'code completion', 'language models', 'language model', 'nlp', 'natural language', 'natural language processing', 'meta-programming', 'story generation', 'story', 'API'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Natural Language :: English",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



import json
import requests


def generate(prompt, temperature, top_p, top_k=40, seed=0, stream=True):
    words = prompt.split(" ")
    assert isinstance(temperature, float), "temperature most be float value"
    assert temperature > 0.1, "temperature must be greater than 0.1"
    assert temperature <= 10.0, "temperature must be less than 10.0"
    assert isinstance(top_p, float), "top_p most be float value"
    assert top_p > 0.0, "top_p must be greater than 0.0"
    assert top_p <= 1.0, "top_p must be less than or equal to 1.0"
    assert isinstance(top_k, int), "top_k most be integer value"
    assert top_k >= 1, "top_k must be greater than or equal to 1"
    assert top_k <= 1000, "top_k must be less than or equal to 1000"
    assert isinstance(seed, int), "seed most be integer value"
    assert seed >= 0, "seed must be greater than or equal to 0"
    assert seed <= 1215752191, "seed must be less than or equal to 1215752191"
    assert isinstance(stream, bool), "stream most be boolean value"
    assert len(words) <= 2048, "prompt must be less than or equal to 2048 words"
    payload = {"prompt": str(prompt), "temperature": temperature, "top_p": top_p, "top_k": top_k, "seed": seed, "stream": stream}
    URL = requests.post("https://bellard.org/textsynth/api/v1/engines/gptj_6B/completions", json=payload)
    text = URL.text

    format_the_text = text.split("\n\n")
    if format_the_text[-1] == '':
        del format_the_text[-1]
    else:
        format_the_text = format_the_text

    create_text = []
    for line in format_the_text:
        json_line = json.loads(line)
        text = json_line["text"]
        create_text.append(text)

    initial_response = "".join(create_text)
    response = initial_response.replace("\n\n", "\n")
    return response.strip()
